/**
 * @author Lluís Moran
 * 
 * TFC Seguretat Informàtica
 * ETIS - UOC
 * 2011
 * 
 */
package cauoc;

import java.io.IOException;
import java.math.BigInteger;
import java.util.Date;
import java.util.List;

import javax.servlet.RequestDispatcher;
import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;



/**
 * Servlet que recull les dades de les tasques 
 * d'administrador i realitza la funció demanada 
 * mitjançant les classes internes de l'aplicació.
 */
@WebServlet("/admin")
public class Admin extends HttpServlet {
	
	protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		String msg = "";
		String desti = "/jsp/menuadmin.jsp";
		boolean error = false; //determina si s'ha de mostrar un missatge d'error o una notificació

		HttpSession session = request.getSession(true);

		if(session.getAttribute("admin") != null && session.getAttribute("admin").equals(true)){
			String opcio = request.getParameter("opcio");

			if(opcio.equals("iniciar")){
				String path = getServletContext().getRealPath("");

				if(Dbops.creardb()){
					msg = "La base de dades s'ha creat correctament. <br>";
				}else{
					msg = "Error: No s'ha pogut crear la base de dades o ja està creada. <br>";
					error = true;
				}

				//Comprova que existeix la CA
				if(Caops.existeixCa(path)){
					msg = msg.concat("La CA ja estava creada.");
				}else{
					msg = msg.concat("Error: La CA no està creada. Heu de fer-ho manualment.");
					error = true;
				}

				if (error){
					request.setAttribute("error", msg);
				}else{
					request.setAttribute("noti", msg);
				}
			}


			if(opcio.equals("reiniciar")){
				String path = getServletContext().getRealPath("");
				error = false;
				if(Dbops.existeixdb()){
					Dbops.esborrardb();
				}
				if(Dbops.creardb()){
					msg = "La base de dades s'ha creat correctament. <br>";
				}else{
					msg = "Error: No s'ha pogut crear la base de dades. <br>";
					error = true;
				}

				if(Caops.existeixCa(path)){
					Caops.esborraCa(path);
					if (Caops.existeixCa(path)){
						msg = msg.concat("Error: No s'ha pogut esborrar la CA.");
						error = true;
					}else{
						msg = msg.concat("S'ha esborrat la CA. Cal que la torneu a crear manualment.");
					}
				}else{
					msg = msg.concat("La CA no existeix. Heu de crear-la manualment.");
				}

				if (error){
					request.setAttribute("error", msg);
				}else{
					msg = msg.concat("<br>La sessió ha estat tancada.");
					request.setAttribute("noti", msg);
				}

				// Tanquem la sessió perque l'usuari procedeixi a crear la CA.
				session.setAttribute("admin", false);
			}

			if(opcio.equals("donaacces")){
				error = false;
				msg = "";
				String stringSerial = request.getParameter("serial");
				// Cal comprovar que és un número
				boolean serialok = true;
				if (!stringSerial.isEmpty()){
					for (int i=0; i<stringSerial.length(); i++){
						if(!Character.isDigit(stringSerial.charAt(i))){
							serialok = false;
						}
					}
				}else{
					serialok = false;
				}

				if (serialok){
					int serial = Integer.parseInt(stringSerial);
					String acces = request.getParameter("acces");

					// Comprovem que el certificat existeix
					List<String[]> al = Dbops.cercaCerts(-1, serial, false, false, (Date)null, (Date)null, "", "", "", "", "", "", "");

					// Si la llista és més petita que 2 només conté la capçalera.
					if (al.size() < 2){
						error = true;
						msg = msg.concat("Error: No hi ha cap certificat amb aquest número de sèrie. <br>");
					}else{
						if(Dbops.esRevocat(serial)){
							error = true;
							msg = msg.concat("Error: Aquest certificat està revocat. <br>");
						}
					}

					if(!error){
						if (acces.equals("si")){
							if(Dbops.ferAdmin(serial)){
								msg = msg.concat("S'ha atorgat accés d'Administrador. <br>");
							}else{
								error = true;
								msg = msg.concat("Error: No s'ha pogut atorgar accés d'Administrador. <br>");
							}
						}else{
							if(Dbops.revocarAdmin(serial)){
								msg = msg.concat("S'ha revocat l'accés d'Administrador. <br>");
							}else{
								error = true;
								msg = msg.concat("Error: No s'ha pogut revocar l'accés d'Administrador. <br>");
							}
						}
					}
				}else{
					error = true;
					msg = msg.concat("Error: El número de sèrie ha de ser un número. <br>");
				}


				desti = "/jsp/gestioadmins.jsp";
				if (error){
					request.setAttribute("error", msg);
				}else{
					request.setAttribute("noti", msg);
				}

			}

			if(opcio.equals("revocar")){
				error = false;
				msg = "";
				String stringRevSerial = request.getParameter("revSerial");
				// Comprovar que és un número
				boolean serialok = true;
				if (!stringRevSerial.isEmpty()){
				for (int i=0; i<stringRevSerial.length(); i++){
					if(!Character.isDigit(stringRevSerial.charAt(i))){
						serialok = false;
					}
				}
				}else{
					serialok = false;
				}

				if (serialok){
					int revSerial = Integer.parseInt(stringRevSerial);
					String motiu = request.getParameter("motiu");
					motiu = motiu.substring(4);
					int revRao = Integer.parseInt(motiu);
					
					// Si la raó de revocació és Remove from CRL (codi 8) es comprova que el certificat
					// estigui retingut (revocat amb motiu Hold)
					if (revRao == 8){
						// Comprovar que el certificat existeix a la CRL i està retingut
						List<String[]> al = Dbops.cercaCRL(revSerial, (Date)null, (Date)null, 6);

						// Si la llista és més petita que 2 només conté la capçalera.
						if (al.size() < 2){
							error = true;
							msg = msg.concat("Error: No hi ha cap certificat retingut a la CRL amb aquest número de sèrie. <br>");
						}else{
							if(Dbops.esRevocat(revSerial)){
								request.setAttribute("rescerca", al);
							}else{
								error = true;
								msg = msg.concat("Error: Aquest certificat no està retingut i per tant no està revocat. <br>");
							}
						}

					}else{
						// Comprovar que el certificat existeix
						List<String[]> al = Dbops.cercaCerts(-1, revSerial, false, false, (Date)null, (Date)null, "", "", "", "", "", "", "");

						// Si la llista és més petita que 2 només conté la capçalera.
						if (al.size() < 2){
							error = true;
							msg = msg.concat("Error: No hi ha cap certificat amb aquest número de sèrie. <br>");
						}else{
							if(!Dbops.esRevocat(revSerial)){
								request.setAttribute("rescerca", al);
							}else{
								error = true;
								msg = msg.concat("Error: Aquest certificat ja està revocat. <br>");
							}
						}
					}

					// Passem els paràmetres de la revocació a la pàgina de confirmació 
					request.setAttribute("revSerial", revSerial);
					request.setAttribute("revRao", revRao);

					// Mostrem les dades del certificat i demanem confirmació
					desti = "/jsp/confrevocacio.jsp";
				}else{
					error = true;
					msg = msg.concat("Error: El número de sèrie ha de ser un número. <br>");
				}

				if (error){
					desti = "/jsp/menurevocacio.jsp";
					request.setAttribute("error", msg);
				}
			}


			if(opcio.equals("confirmarevocar")){
				String path = getServletContext().getRealPath("");
				msg = "";

				int intRevSerial = Integer.parseInt(request.getParameter("revSerial"));
				BigInteger revSerial = BigInteger.valueOf(intRevSerial);
				String motiu = request.getParameter("revRao");
				int revRao = Integer.parseInt(motiu);
				Caops.revocaCert(path, revSerial, revRao);

				msg = msg.concat("El certificat ha estat revocat.");
				request.setAttribute("noti", msg);
			}
			
			if(opcio.equals("publicrl")){
				String path = getServletContext().getRealPath("");
				msg = "";

				if(Caops.publicaCRL(path)){
					msg = msg.concat("La nova Llista de Revocació de Certificats s'ha publicat correctament.");
				}else{
					error = true;
					msg = msg.concat("Hi ha hagut un error en publicar la Llista de Revocació de Certificats.");
				}

				desti = "/jsp/menuadmin.jsp";
				if (error){
					request.setAttribute("error", msg);
				}else{
					request.setAttribute("noti", msg);
				}
				
			}

		}else{
			desti = "/jsp/sessionerror.jsp";
		}

		RequestDispatcher rd = getServletContext().getRequestDispatcher(desti);
		rd.forward(request, response);

	}

}		
