/**
 * @author Lluís Moran
 * 
 * TFC Seguretat Informàtica
 * ETIS - UOC
 * 2011
 * 
 */
package cauoc;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.math.BigInteger;
import java.security.InvalidKeyException;
import java.security.KeyFactory;
import java.security.KeyPair;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.Security;
import java.security.SignatureException;
import java.security.cert.CertificateException;
import java.security.cert.CertificateExpiredException;
import java.security.cert.CertificateNotYetValidException;
import java.security.cert.CertificateParsingException;
import java.security.cert.X509Certificate;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.PKCS8EncodedKeySpec;
import java.util.Calendar;
import java.util.Date;
import java.util.Enumeration;
import java.util.GregorianCalendar;

import org.bouncycastle.asn1.ASN1ObjectIdentifier;
import org.bouncycastle.asn1.ASN1Set;
import org.bouncycastle.asn1.DERIA5String;
import org.bouncycastle.asn1.DERSequence;
import org.bouncycastle.asn1.pkcs.Attribute;
import org.bouncycastle.asn1.pkcs.CertificationRequestInfo;
import org.bouncycastle.asn1.pkcs.PKCSObjectIdentifiers;
import org.bouncycastle.asn1.x500.X500Name;
import org.bouncycastle.asn1.x509.BasicConstraints;
import org.bouncycastle.asn1.x509.CRLNumber;
import org.bouncycastle.asn1.x509.DistributionPoint;
import org.bouncycastle.asn1.x509.DistributionPointName;
import org.bouncycastle.asn1.x509.ExtendedKeyUsage;
import org.bouncycastle.asn1.x509.GeneralName;
import org.bouncycastle.asn1.x509.GeneralNames;
import org.bouncycastle.asn1.x509.KeyPurposeId;
import org.bouncycastle.asn1.x509.KeyUsage;
import org.bouncycastle.asn1.x509.SubjectPublicKeyInfo;
import org.bouncycastle.asn1.x509.X509Extension;
import org.bouncycastle.asn1.x509.X509Extensions;
import org.bouncycastle.cert.X509CRLHolder;
import org.bouncycastle.cert.X509CertificateHolder;
import org.bouncycastle.cert.X509v2CRLBuilder;
import org.bouncycastle.cert.X509v3CertificateBuilder;
import org.bouncycastle.cert.jcajce.JcaX509CertificateConverter;
import org.bouncycastle.jce.PKCS10CertificationRequest;
import org.bouncycastle.jce.X509Principal;
import org.bouncycastle.jce.provider.BouncyCastleProvider;
import org.bouncycastle.jce.provider.JDKKeyPairGenerator;
import org.bouncycastle.operator.ContentSigner;
import org.bouncycastle.operator.OperatorCreationException;
import org.bouncycastle.operator.jcajce.JcaContentSignerBuilder;
import org.bouncycastle.x509.extension.AuthorityKeyIdentifierStructure;
import org.bouncycastle.x509.extension.SubjectKeyIdentifierStructure;


/**
 * Classe que proporciona les funcions necessàries
 * per realitzar operacions amb l'Autoritat de Certificació
 */
public class Caops {

	static final String CA_PATH = "/ca/";
	static final String PRIV_PATH = "/ca/privat/";
	static final String CERT_PATH = "/ca/certificats/";
	static final String CRL_PATH = "/ca/crl/";
	static final String PUB_PATH = "/pub/";	
	static final String TOMCAT_PATH = "/tomcat/";
	static final String SERIAL = "serial";
	static final char[] CA_PWD = LoadConfig.CA_PWD;
	static final String CA_KS = "ca.jks";
	static final String CA_ALIAS = "ca";

	/**
	 * Mètode que esborra tots els fitxers de la CA
	 * per tal de reiniciar-la manualment.
	 * 
	 * @param absPath Ruta de l'aplicació 
	 */
	public static void esborraCa(String absPath){

		File dir = new File(absPath + CA_PATH);
		File[] fitxers = dir.listFiles();
		for (int i=0;i<fitxers.length;i++){
			if (fitxers[i].isFile()) {
				fitxers[i].delete();
			}
		}

		dir = new File(absPath + CERT_PATH);
		fitxers = dir.listFiles();
		for (int i=0;i<fitxers.length;i++){
			if (fitxers[i].isFile()) {
				fitxers[i].delete();
			}
		}

		dir = new File(absPath + CRL_PATH);
		fitxers = dir.listFiles();
		for (int i=0;i<fitxers.length;i++){
			if (fitxers[i].isFile()) {
				fitxers[i].delete();
			}
		}

		dir = new File(absPath + PRIV_PATH);
		fitxers = dir.listFiles();
		for (int i=0;i<fitxers.length;i++){
			if (fitxers[i].isFile()) {
				fitxers[i].delete();
			}
		}
		
		dir = new File(absPath + PUB_PATH);
		fitxers = dir.listFiles();
		for (int i=0;i<fitxers.length;i++){
			if (fitxers[i].isFile()) {
				fitxers[i].delete();
			}
		}
		
		dir = new File(absPath + TOMCAT_PATH);
		fitxers = dir.listFiles();
		for (int i=0;i<fitxers.length;i++){
			if (fitxers[i].isFile()) {
				fitxers[i].delete();
			}
		}

	}

	/**
	 * Mètode que comprova si existeix la CA.
	 * Per fer-ho comprova que existeix el keystore i la clau privada de la CA.
	 * 
	 * @param absPath Ruta de l'aplicació 
	 * @return True si la CA existeix
	 */
	public static boolean existeixCa(String absPath){
		boolean ret = false;
		Security.addProvider(new BouncyCastleProvider());

		KeyStore ks = null;
		ks = loadks(absPath);

		PrivateKey privKey = loadpk(absPath);
		
		try {
			if(ks.containsAlias(CA_ALIAS) && privKey != null){
				ret = true;
			}
		} catch (KeyStoreException e) {
			// No fem res. Es retornarà fals.
		} catch (NullPointerException e){
			// No fem res. Es retornarà fals.
		}

		return ret;
	}


	/**
	 * Mètode que rep una petició de certificació 
	 * que es signarà per la CA i generarà el certificat resultant.
	 * Retornarà el path del fitxer del certificat o
	 * un missatge d'error en cas que hi hagi algun problema.
	 * Cal que la petició de certificació estigui en format DER.
	 * 
	 * @param absPath Path absolut del Servlet
	 * @param is InputStream amb la petició de certificació
	 * @return El Path complert del fitxer del certificat 
	 * 		   o un missatge d'error si no s'ha pogut generar el certificat
	 */
	public static String signapkcs10(String absPath, InputStream is) {

		String ret = "";
		Security.addProvider(new BouncyCastleProvider());

		byte[] buffer = istobytes(is);

		// Cal que la petició de certificació estigui en format DER
		try{
			if ((int)buffer.length > 0){
				PKCS10CertificationRequest csr = new PKCS10CertificationRequest(buffer);
				if(csr.verify()){
					// Recolectem les dades de la petició
					PublicKey subjectPubKey = csr.getPublicKey("BC");
					CertificationRequestInfo cri = csr.getCertificationRequestInfo();
					cri.getSubject().getDEREncoded();					
					
					X509Principal subjectPrincipal = new X509Principal(X500Name.getInstance(cri.getSubject()).toString());
					X500Name subjectName = new X500Name(subjectPrincipal.getName());

					SubjectPublicKeyInfo spi = cri.getSubjectPublicKeyInfo();

					// Recolectem les dades de la CA
					KeyStore ks = loadks(absPath);
					X509Certificate cacert = (X509Certificate) ks.getCertificate(CA_ALIAS);

					PrivateKey cakey = loadpk(absPath);
					
					X509Principal issuerPrincipal = new X509Principal(true, cacert.getIssuerX500Principal().getName());
					X500Name issuerName = new X500Name(issuerPrincipal.getName());

					// Cerquem l'últim serial que hem fet servir
					BigInteger serial = BigInteger.valueOf(Dbops.getNextSerial());

					// Carreguem la validesa del certificat de la configuració
					GregorianCalendar cal = new GregorianCalendar();
					Date notBefore = cal.getTime();
					cal.set(Calendar.DAY_OF_YEAR, LoadConfig.CERTDAYS + cal.get(Calendar.DAY_OF_YEAR));
					Date notAfter = cal.getTime();

					// Creem el certificat a signar
					X509v3CertificateBuilder certbuilder = new X509v3CertificateBuilder(issuerName, serial, notBefore, notAfter, subjectName, spi);

					// Afegim extensions
					certbuilder.addExtension(X509Extension.authorityKeyIdentifier, false, new AuthorityKeyIdentifierStructure(cacert));
					certbuilder.addExtension(X509Extension.subjectKeyIdentifier, false, new SubjectKeyIdentifierStructure(subjectPubKey));
					certbuilder.addExtension(X509Extension.basicConstraints, true, new BasicConstraints(false));
					certbuilder.addExtension(X509Extension.keyUsage, true, new KeyUsage(KeyUsage.digitalSignature | KeyUsage.keyEncipherment | KeyUsage.keyAgreement));
					certbuilder.addExtension(X509Extension.extendedKeyUsage, false, new ExtendedKeyUsage(KeyPurposeId.id_kp_clientAuth));
					
					// Afegim CRL Distribution Point
					String crlURL = "http://" + LoadConfig.HOST + "/pub/ca.crl";
					GeneralName gn = new GeneralName(GeneralName.uniformResourceIdentifier,new DERIA5String(crlURL));
					GeneralNames gns = new GeneralNames(new DERSequence(gn));
					DistributionPointName dpn = new DistributionPointName(0, gns);
					DistributionPoint distp = new DistributionPoint(dpn, null, null);
					certbuilder.addExtension(X509Extension.cRLDistributionPoints, false, new DERSequence(distp));
					


					// Extreiem extensions del CRS
					if (cri.getAttributes() != null){
						ASN1Set atributs = cri.getAttributes();
						for (int i = 0; i != atributs.size(); i++){
							Attribute atri = Attribute.getInstance(atributs.getObjectAt(i));
							if(atri.getAttrType().equals(PKCSObjectIdentifiers.pkcs_9_at_extensionRequest)){
								X509Extensions extensions = X509Extensions.getInstance(atri.getAttrValues().getObjectAt(0));
								Enumeration e = extensions.oids();
								while (e.hasMoreElements()){
									ASN1ObjectIdentifier oid = (ASN1ObjectIdentifier)e.nextElement();
									X509Extension ext = extensions.getExtension(oid);
									certbuilder.addExtension(oid, ext.isCritical(), ext.getValue());
								}
							}
						}
					}

					ContentSigner cs = new JcaContentSignerBuilder("SHA1withRSA").setProvider("BC").build(cakey);
					X509CertificateHolder certHolder = certbuilder.build(cs);

					// Guardem el certificat en un objecte X509Certificate
					// En realitat només es fa servir per extreure el sigAlgName
					JcaX509CertificateConverter converter = new JcaX509CertificateConverter().setProvider("BC");
					X509Certificate subjectCert = converter.getCertificate(certHolder);

					// Guardem les dades del certificat a la base de dades
					int cerVersion = certHolder.getVersion();
					int cerSerial = certHolder.getSerialNumber().intValue();
					String cerSAN = subjectCert.getSigAlgName();
					String cerIN = certHolder.getIssuer().toString();
					Date cerNB = certHolder.getNotBefore();
					Date cerNA = certHolder.getNotAfter();
					String cerSN = certHolder.getSubject().toString();

					if(Dbops.guardaCertificat(cerVersion, cerSerial, cerSAN, cerIN, cerNB, cerNA, cerSN)){
						// Guardem el certificat en un fitxer
						FileOutputStream fos = new FileOutputStream(absPath + CERT_PATH + cerSerial + ".cer");
						fos.write(certHolder.getEncoded());
						ret = absPath + CERT_PATH + cerSerial + ".cer";
					}else{
						ret = ret.concat("Error: No s'ha pogut guardar el certificat a la base de dades. <br>");
					}
				}else{
					ret = ret.concat("Error: No es pot verificar la petició de certificat. <br>");
				}
			}else{
				ret = ret.concat("Error: No heu escollit cap fitxer o el fitxer no és correcte. <br>");
			}
		} catch (InvalidKeyException e) {
			ret = ret.concat("Error: Hi ha hagut un problema amb la clau pública. <br>");
		} catch (CertificateParsingException e) {
			ret = ret.concat("Error: El fitxer no conté una petició de certificació en format DER. <br>");
		} catch (NoSuchAlgorithmException e) {
			ret = ret.concat("Error: Hi ha hagut un problema amb l'algoritme de la clau pública. <br>");
		} catch (NoSuchProviderException e) {
			ret = ret.concat("Error: No s'ha pogut carregar el proveïdor criptogràfic. <br>");
		} catch (SignatureException e) {
			ret = ret.concat("Error: La signatura de la vostra petició no és correcta. <br>");
		} catch (KeyStoreException e) {
			ret = ret.concat("Error: Hi ha hagut un problema amb el magatzem de la CA. Verifiqueu que existeix. <br>");
		} catch (OperatorCreationException e) {
			ret = ret.concat("Error: Hi ha hagut un problema en signar la vostra petició. <br>");
		} catch (CertificateException e) {
			ret = ret.concat("Error: Hi ha hagut un problema amb els certificats <br>");
		} catch (FileNotFoundException e) {
			ret = ret.concat("Error: No s'ha pogut guardar el vostre certificat signat. <br>");
		} catch (IOException e) {
			ret = ret.concat("Error: No s'ha pogut guardar el vostre certificat signat. <br>");
		} catch (IllegalArgumentException e) {
			ret = ret.concat("Error: El fitxer no conté una petició de certificació en format DER. <br>");
		}

		return ret;
	}

	/**
	 * Mètode que genera un fitxer PKCS#12 a partir de les dades de l'usuari.
	 * Aquest fitxer contindrà una cadena de certificats amb el certificat de la CA,
	 * el certificat de l'usuari signat per la CA i la clau privada de l'usuari. 
	 * Retornarà el path del fitxer PKCS#12 o
	 * un missatge d'error en cas que hi hagi algun problema.
	 * 
	 * @param absPath Path absolut del Servlet
	 * @param is InputStream amb la petició de certificació
	 * @return El Path complert del fitxer del certificat 
	 * 		   o un missatge d'error si no s'ha pogut generar el certificat
	 */
	public static String generapkcs12(String subjectDN, String password, String absPath){

		String ret = "";
		Security.addProvider(new BouncyCastleProvider());
		try{
			// Generem les claus i guardem les dades de l'usuari
			JDKKeyPairGenerator keyGen = new JDKKeyPairGenerator.RSA();
			keyGen.initialize(2048);
			KeyPair clausSubject = keyGen.generateKeyPair();

			PublicKey subjectPubKey = clausSubject.getPublic();
			PrivateKey subjectPrivKey = clausSubject.getPrivate();
			X509Principal subjectPrincipal = new X509Principal(true, subjectDN);
			X500Name subjectName = new X500Name(subjectPrincipal.getName());
			char[] subjectPwd = password.toCharArray();

			//Generem una petició de certificació
			PKCS10CertificationRequest csr = new PKCS10CertificationRequest("SHA1withRSA", subjectPrincipal, subjectPubKey, null, clausSubject.getPrivate());
			CertificationRequestInfo cri = csr.getCertificationRequestInfo();
			cri.getSubject().getDEREncoded();
			SubjectPublicKeyInfo spi = cri.getSubjectPublicKeyInfo();

			// Recolectem les dades de la CA
			KeyStore ks = loadks(absPath);
			X509Certificate cacert = (X509Certificate) ks.getCertificate(CA_ALIAS);

			PrivateKey cakey = loadpk(absPath);
			
			X509Principal issuerPrincipal = new X509Principal(true, cacert.getIssuerX500Principal().getName());
			X500Name issuerName = new X500Name(issuerPrincipal.getName());
			
			// Cerquem l'últim serial que hem fet servir
			BigInteger serial = BigInteger.valueOf(Dbops.getNextSerial());

			// Dates de validesa del certificat
			GregorianCalendar cal = new GregorianCalendar();
			Date notBefore = cal.getTime();
			cal.set(Calendar.DAY_OF_YEAR, LoadConfig.CERTDAYS + cal.get(Calendar.DAY_OF_YEAR));
			Date notAfter = cal.getTime();

			// Creem el certificat a signar
			X509v3CertificateBuilder certbuilder = new X509v3CertificateBuilder(issuerName, serial, notBefore, notAfter, subjectName, spi);

			// Afegim extensions
			certbuilder.addExtension(X509Extension.authorityKeyIdentifier, false, new AuthorityKeyIdentifierStructure(cacert));
			certbuilder.addExtension(X509Extension.subjectKeyIdentifier, false, new SubjectKeyIdentifierStructure(subjectPubKey));
			certbuilder.addExtension(X509Extension.basicConstraints, true, new BasicConstraints(false));
			certbuilder.addExtension(X509Extension.keyUsage, true, new KeyUsage(KeyUsage.digitalSignature | KeyUsage.keyEncipherment | KeyUsage.keyAgreement));
			certbuilder.addExtension(X509Extension.extendedKeyUsage, false, new ExtendedKeyUsage(KeyPurposeId.id_kp_clientAuth));
			
			// Afegim CRL Distribution Point
			String crlURL = "http://" + LoadConfig.HOST + "/pub/ca.crl";
			GeneralName gn = new GeneralName(GeneralName.uniformResourceIdentifier,new DERIA5String(crlURL));
			GeneralNames gns = new GeneralNames(new DERSequence(gn));
			DistributionPointName dpn = new DistributionPointName(0, gns);
			DistributionPoint distp = new DistributionPoint(dpn, null, null);
			certbuilder.addExtension(X509Extension.cRLDistributionPoints, false, new DERSequence(distp));

			// Extreiem extensions del CRS
			if (cri.getAttributes() != null){
				ASN1Set atributs = cri.getAttributes();
				for (int i = 0; i != atributs.size(); i++){
					Attribute atri = Attribute.getInstance(atributs.getObjectAt(i));
					if(atri.getAttrType().equals(PKCSObjectIdentifiers.pkcs_9_at_extensionRequest)){
						X509Extensions extensions = X509Extensions.getInstance(atri.getAttrValues().getObjectAt(0));
						Enumeration e = extensions.oids();
						while (e.hasMoreElements()){
							ASN1ObjectIdentifier oid = (ASN1ObjectIdentifier)e.nextElement();
							X509Extension ext = extensions.getExtension(oid);
							certbuilder.addExtension(oid, ext.isCritical(), ext.getValue());
						}
					}
				}
			}

			ContentSigner cs = new JcaContentSignerBuilder("SHA1withRSA").setProvider("BC").build(cakey);
			X509CertificateHolder certHolder = certbuilder.build(cs);
			
			// Creem un Keystore PKCS12
			KeyStore ks12 = KeyStore.getInstance("PKCS12", "BC");
			ks12.load(null, null);

			X509Certificate[] cadenaCerts = new X509Certificate[2];

			JcaX509CertificateConverter converter = new JcaX509CertificateConverter().setProvider("BC");
			X509Certificate subjectCert = converter.getCertificate(certHolder);

			cadenaCerts[0] = subjectCert;
			cadenaCerts[1] = cacert;

			ks12.setKeyEntry("cert", subjectPrivKey, subjectPwd, cadenaCerts);

			// Guardem les dades del certificat a la base de dades
			int cerVersion = certHolder.getVersion();
			int cerSerial = certHolder.getSerialNumber().intValue();
			String cerSAN = subjectCert.getSigAlgName();
			String cerIN = certHolder.getIssuer().toString();
			Date cerNB = certHolder.getNotBefore();
			Date cerNA = certHolder.getNotAfter();
			String cerSN = certHolder.getSubject().toString();

			if(Dbops.guardaCertificat(cerVersion, cerSerial, cerSAN, cerIN, cerNB, cerNA, cerSN)){
				// Guardem el pkcs12 en un fitxer
				FileOutputStream fos = new FileOutputStream(absPath + CERT_PATH + cerSerial + ".p12");
				ks12.store(fos, subjectPwd);
				ret = absPath + CERT_PATH + cerSerial + ".p12";
			}else{
				ret = ret.concat("Error: No s'ha pogut guardar el certificat a la base de dades. <br>");
			}

		} catch (InvalidKeyException e) {
			ret = ret.concat("Error: Hi ha hagut un problema amb la clau pública. <br>");
		} catch (NoSuchAlgorithmException e) {
			ret = ret.concat("Error: Hi ha hagut un problema amb l'algoritme de la clau pública. <br>");
		} catch (NoSuchProviderException e) {
			ret = ret.concat("Error: No s'ha pogut carregar el proveïdor criptogràfic. <br>");
		} catch (SignatureException e) {
			ret = ret.concat("Error: La signatura de la vostra petició no és correcta. <br>");
		} catch (KeyStoreException e) {
			ret = ret.concat("Error: Hi ha hagut un problema amb el magatzem de la CA. Verifiqueu que existeix. <br>");
		} catch (OperatorCreationException e) {
			ret = ret.concat("Error: Hi ha hagut un problema en signar la vostra petició. <br>");
		} catch (CertificateException e) {
			ret = ret.concat("Error: Hi ha hagut un problema amb els certificats <br>");
		} catch (IOException e) {
			ret = ret.concat("Error: No s'ha pogut guardar el vostre certificat signat. <br>");
		}
		return ret;

	}
	
	/**
	 * Mètode que revoca un certificat, afegint-lo a la CRL i publicant aquesta.
	 * 
	 * @param absPath absPath Path absolut del Servlet
	 * @param revSerial Objecte BigInteger que conté el número del certificat a revocar
	 * @param revMotiu Objecte int que conté el codi del motiu de revocació segons el RFC5280
	 * @return True si s'ha pogut dur a terme la revocació
	 */
	public static boolean revocaCert(String absPath, BigInteger revSerial, int revMotiu){
		Security.addProvider(new BouncyCastleProvider());
		boolean ret = true;
		
		try{
			// Recolectem les dades de la CA i de la CRL
			KeyStore ks = loadks(absPath);
			X509Certificate cacert = (X509Certificate) ks.getCertificate(CA_ALIAS);
			PrivateKey cakey = loadpk(absPath);
			X500Name issuerName = new X500Name(cacert.getSubjectX500Principal().getName());
			X509CRLHolder crlAnterior = loadCRL(absPath);

			GregorianCalendar cal = new GregorianCalendar();
			Date thisUpdate = cal.getTime();
			cal.set(Calendar.DAY_OF_YEAR, LoadConfig.CRLDAYS + cal.get(Calendar.DAY_OF_YEAR));
			Date nextUpdate = cal.getTime();

			X509v2CRLBuilder crlBuilder = new X509v2CRLBuilder(issuerName, thisUpdate);
			crlBuilder.addCRL(crlAnterior);
			crlBuilder.setNextUpdate(nextUpdate);
			crlBuilder.addExtension(X509Extension.authorityKeyIdentifier, false, new AuthorityKeyIdentifierStructure(cacert));
			
			//Recuperem el següent número de sèrie de CRL de la base de dades
			int crlnumber = Dbops.getCRLNextSerial();
			crlBuilder.addExtension(X509Extension.cRLNumber, false, new CRLNumber(BigInteger.valueOf(crlnumber)));
			
			//Afegim el certificat
			crlBuilder.addCRLEntry(revSerial, thisUpdate, revMotiu);

			ContentSigner cs = new JcaContentSignerBuilder("SHA1withRSA").setProvider("BC").build(cakey);
			X509CRLHolder CRLHolder = crlBuilder.build(cs);

			//Guarda a la Base de Dades
			if(!Dbops.actualitzaCRL(crlnumber, CRLHolder.getIssuer().toString(), thisUpdate, nextUpdate)){
				ret = false;
			}
			if(!Dbops.guardaCertCRL(revSerial.intValue(), thisUpdate, revMotiu)){
				ret = false;
			}

			// Guardem la CRL en un fitxer
			if(ret){
				saveCRL(absPath, CRLHolder);
			}
			
		} catch (KeyStoreException e){
			// Retornem false
			ret = false;
		} catch (CertificateParsingException e) {
			// Retornem false
			ret = false;
		} catch (OperatorCreationException e) {
			// Retornem false
			ret = false;
		}
		
		return ret;
	}
	
	/**
	 * Mètode que publica una nova llista de revocació de certificats (CRL).
	 * 
	 * @param absPath absPath Path absolut del Servlet
	 * @return True si s'ha pogut dur a terme la publicació de la nova CRL
	 */
	public static boolean publicaCRL(String absPath){
		Security.addProvider(new BouncyCastleProvider());
		boolean ret = true;
		
		try{
			// Recolectem les dades de la CA i de la CRL
			KeyStore ks = loadks(absPath);
			X509Certificate cacert = (X509Certificate) ks.getCertificate(CA_ALIAS);
			PrivateKey cakey = loadpk(absPath);
			X500Name issuerName = new X500Name(cacert.getSubjectX500Principal().getName());
			X509CRLHolder crlAnterior = loadCRL(absPath);
			
			GregorianCalendar cal = new GregorianCalendar();
			Date thisUpdate = cal.getTime();
			cal.set(Calendar.DAY_OF_YEAR, LoadConfig.CRLDAYS + cal.get(Calendar.DAY_OF_YEAR));
			Date nextUpdate = cal.getTime();

			X509v2CRLBuilder crlBuilder = new X509v2CRLBuilder(issuerName, thisUpdate);
			crlBuilder.addCRL(crlAnterior);
			crlBuilder.setNextUpdate(nextUpdate);
			crlBuilder.addExtension(X509Extension.authorityKeyIdentifier, false, new AuthorityKeyIdentifierStructure(cacert));
			//Recuperem el següent número de sèrie de CRL de la base de dades
			int crlnumber = Dbops.getCRLNextSerial();
			crlBuilder.addExtension(X509Extension.cRLNumber, false, new CRLNumber(BigInteger.valueOf(crlnumber)));

			ContentSigner cs = new JcaContentSignerBuilder("SHA1withRSA").setProvider("BC").build(cakey);
			X509CRLHolder CRLHolder = crlBuilder.build(cs);

			// Guarda a la Base de Dades
			if(!Dbops.actualitzaCRL(crlnumber, CRLHolder.getIssuer().toString(), thisUpdate, nextUpdate)){
				ret = false;
			}

			if(ret){
				// Guardem la CRL en un fitxer
				saveCRL(absPath, CRLHolder);
			}

			
		} catch (KeyStoreException e){
			// Retornem false
			ret = false;
		} catch (CertificateParsingException e) {
			// Retornem false
			ret = false;
		} catch (OperatorCreationException e) {
			// Retornem false
			ret = false;
		}
		
		return ret;
	}
	
	/**
	 * Mètode que valida un certificat com a Administrador. 
	 * Per fer-ho comprova que està signat per la CA, que no està caducat,
	 * que no està revocat i que té drets d'administrador.
	 * Cal que el certificat estigui en format DER.
	 * 
	 * @param absPath absPath Path absolut del Servlet
	 * @param is InputStream amb el certificat de l'usuari en format DER
	 * @return String que conté un error o la cadena "Login ok" si s'ha pogut verificar el certificat
	 */
	public static String adminCert(String absPath, X509Certificate adminCert) {

		String ret = "";
		Security.addProvider(new BouncyCastleProvider());

		try{
				int cerSerial = adminCert.getSerialNumber().intValue();

				// Recolectem les dades de la CA
				KeyStore ks = loadks(absPath);
				X509Certificate cacert = (X509Certificate) ks.getCertificate(CA_ALIAS);
				
				// Validem el certificat. Si hi ha un error l'excepció enviarà un missatge.
				adminCert.verify(cacert.getPublicKey(), "BC");
				adminCert.checkValidity();
				
				// Mirem si el certifcat és revocat i si té drets d'administrador
				if(Dbops.esRevocat(cerSerial)){
					ret = ret.concat("Error: El certificat està revocat. <br>");
				}
				if(!Dbops.esAdmin(cerSerial)){
					ret = ret.concat("Error: El certificat no té drets d'administrador. <br>");
				}
				
				if(ret == null || !ret.startsWith("Error")){
					ret = ret.concat("Login ok");
				}

		} catch (CertificateParsingException e) {
			ret = ret.concat("Error: El fitxer no conté un certificat en format DER. <br>");
		} catch (NoSuchProviderException e) {
			ret = ret.concat("Error: No s'ha pogut carregar el proveïdor criptogràfic. <br>");
		} catch (KeyStoreException e) {
			ret = ret.concat("Error: Hi ha hagut un problema amb el magatzem de la CA. Verifiqueu que existeix. <br>");
		} catch (InvalidKeyException e) {
			ret = ret.concat("Error: El certificat no és vàlid. La CA no l'ha signat. <br>");
		} catch (NoSuchAlgorithmException e) {
			ret = ret.concat("Error: Hi ha hagut un problema amb l'algorisme criptogràfic del certificat. <br>");
		} catch (SignatureException e) {
			ret = ret.concat("Error: Hi ha hagut un problema amb la signatura. <br>");
		} catch (CertificateExpiredException e) {
			ret = ret.concat("Error: El certificat ha caducat. <br>");
		} catch (CertificateNotYetValidException e) {
			ret = ret.concat("Error: El certificat no ha arribat a la seva data de validesa. <br>");
		} catch (CertificateException e) {
			ret = ret.concat("Error: Hi ha hagut un problema amb el certificat. <br>");
		}

		return ret;
	}


	/**
	 * Mètode privat que transforma un InputStream en una cadena de bytes
	 * 
	 * @param is InputStream a transformar
	 * @return Una cadena de bytes amb la informació de l'InputStream
	 */
	private static byte[] istobytes(InputStream is) {
		// Passa l'InputStream a bytes[]
		int len;
		int size = 1024;
		byte[] buffer = null;

		try {
			if (is instanceof ByteArrayInputStream) {
				size = is.available();
				buffer = new byte[size];
				len = is.read(buffer, 0, size);
			} else {
				ByteArrayOutputStream bos = new ByteArrayOutputStream();
				buffer = new byte[size];
				while ((len = is.read(buffer, 0, size)) != -1)
					bos.write(buffer, 0, len);
				buffer = bos.toByteArray();
			}
		} catch (IOException e) {
			e.printStackTrace();
		}

		return buffer;
	}

	/**
	 * Métode que carrega el magatzem de claus de la CA per accedir al seu certificat.
	 * Retornarà null en cas que no es pugui carregar el magatzem.
	 * Els métodes que el cridin hauran d'encarregar-se de notificar l'error.
	 * 
	 * @param absPath Ruta de l'aplicació 
	 * @return Objecte KeyStore que conté el magatzem de claus de la CA o null si no s'aconssegueix
	 */
	private static KeyStore loadks(String absPath) {
		FileInputStream fis = null;
		KeyStore ks = null;
		try {
			fis = new FileInputStream(absPath + CA_PATH + "ca.jks");
			ks = KeyStore.getInstance("JKS");
			ks.load(fis, CA_PWD);
			fis.close();	
		} catch (FileNotFoundException e) {
			// No fem res. Ja retorna null
		} catch (KeyStoreException e) {
			// No fem res. Ja retorna null
		} catch (NoSuchAlgorithmException e) {
			// No fem res. Ja retorna null
		} catch (CertificateException e) {
			// No fem res. Ja retorna null
		} catch (IOException e) {
			// No fem res. Ja retorna null
		}finally {
			if (fis != null) {
				try {
					fis.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}

		return ks;
	}

	/**
	 * Métode que carrega la clau privada de la CA.
	 * Retornarà null en cas que no es pugui carregar la clau.
	 * Els métodes que el cridin hauran d'encarregar-se de notificar l'error.
	 * 
	 * @param absPath Ruta de l'aplicació 
	 * @return Objecte PrivateKey que conté la clau privada de la CA o null si no s'aconssegueix
	 */
	private static PrivateKey loadpk(String absPath) {
		PrivateKey cakey = null;
		FileInputStream fis;
		try {
			File filePrivateKey = new File(absPath + PRIV_PATH + "cakey.der");
			fis = new FileInputStream(absPath + PRIV_PATH + "cakey.der");
			byte[] encodedPrivateKey = new byte[(int) filePrivateKey.length()];
			fis.read(encodedPrivateKey);
			fis.close();

			KeyFactory keyFactory = KeyFactory.getInstance("RSA");
			PKCS8EncodedKeySpec privateKeySpec = new PKCS8EncodedKeySpec(encodedPrivateKey);
			cakey = keyFactory.generatePrivate(privateKeySpec);

		} catch (FileNotFoundException e) {
			// No fem res. Ja retorna null
		} catch (IOException e) {
			// No fem res. Ja retorna null
		} catch (NoSuchAlgorithmException e) {
			// No fem res. Ja retorna null
		} catch (InvalidKeySpecException e) {
			// No fem res. Ja retorna null
		}

		return cakey;
	}
	
	/**
	 * Métode que carrega el fitxer de la llista de revocació de certificats (CRL).
	 * Retornarà null en cas que no es pugui carregar la llista.
	 * Els métodes que el cridin hauran d'encarregar-se de notificar l'error.
	 * 
	 * @param absPath Ruta de l'aplicació 
	 * @return Objecte X509CRLHolder que conté la llista de revocació de certificats (CRL)
	 *         o null si no s'aconssegueix
	 */
	private static X509CRLHolder loadCRL(String absPath) {
		X509CRLHolder caCRL = null;
		FileInputStream fis;
		try {
			File fileCRL = new File(absPath + CRL_PATH + "ca.crl");
			fis = new FileInputStream(absPath + CRL_PATH + "ca.crl");
			byte[] encodedCRL = new byte[(int) fileCRL.length()];
			fis.read(encodedCRL);
			fis.close();

			caCRL = new X509CRLHolder (encodedCRL);

		} catch (FileNotFoundException e) {
			// No fem res. Ja retorna null
		} catch (IOException e) {
			// No fem res. Ja retorna null
		}

		return caCRL;
	}

	/**
	 * Métode que guarda en un fitxer la llista de revocació de 
	 * certificats (CRL) i en fa una còpia a la carpeta pública 
	 * perquè es pugui descarregar.
	 * 
	 * @param absPath Ruta de l'aplicació
	 * @param crlHolder Llista de revocació a guardar
	 */
	private static void saveCRL(String absPath, X509CRLHolder crlHolder){
		FileOutputStream fos;
		try {
			fos = new FileOutputStream(absPath + CRL_PATH + "ca.crl");
			fos.write(crlHolder.getEncoded());
			fos.close();

			//Fem una còpia a pub per descarregar
			fos = new FileOutputStream(absPath + PUB_PATH + "ca.crl");
			fos.write(crlHolder.getEncoded());
			fos.close();

		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

}
