/**
 * @author Lluís Moran
 * 
 * TFC Seguretat Informàtica
 * ETIS - UOC
 * 2011
 * 
 */
package cauoc;

import java.io.IOException;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;

import javax.servlet.RequestDispatcher;
import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;



/**
 * Servlet que recull les dades del menú de consultes
 * i realitza la funció demanada mitjançant les
 * classes internes de l'aplicació.
 */
@WebServlet("/consulta")
public class Consulta extends HttpServlet {
	
	protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		String msg = "";
		String desti = "/jsp/menuconsulta.jsp";
		boolean error = false; //determina si s'ha de mostrar un missatge d'error o una notificació

		HttpSession session = request.getSession(true);

		if(session.getAttribute("admin") != null && session.getAttribute("admin").equals(true)){
			String opcio = request.getParameter("opcio");

			if(opcio.equals("cercacerts")){
				// Declarem paràmetres de la cerca
				// Els camps que no volguem fer servir els passarem buits si són Strings,
				// negatius si són int, null si són Date o false si són Boolean.
				int qversio = -1;
				int qserial = -1;
				boolean qadmin = false;
				boolean qrev = false;
				Date qnb = null;
				Date qna = null;

				// La resta de paràmetres es passaran com a String			

				// Recollim i comprovem els paràmetres de cerca
				String versio = request.getParameter("versio");
				String serial = request.getParameter("serial");
				String admin = request.getParameter("admin");
				String revocat = request.getParameter("revocat");
				String nbdia = request.getParameter("nbdia");
				String nbmes = request.getParameter("nbmes");
				String nbany = request.getParameter("nbany");
				String nadia = request.getParameter("nadia");
				String names = request.getParameter("names");
				String naany = request.getParameter("naany");
				String country = request.getParameter("C");
				String st = request.getParameter("ST");
				String loc = request.getParameter("L");
				String org = request.getParameter("O");
				String orgunit = request.getParameter("OU");
				String cn = request.getParameter("CN");
				String email = request.getParameter("EMAILADDRESS");

				if (!versio.isEmpty()){
					if(Character.isDigit(versio.charAt(0))){
						qversio = Integer.parseInt(versio);
						if (qversio < 1 || qversio > 3){
							// Error la versió ha de ser un número entre 1 i 3
							error = true;
							msg = msg.concat("Error: La versió ha de ser un número entre 1 i 3.<br>");
							
						}
					}else{
						// Error la versió ha de ser un número
						error = true;
						msg = msg.concat("Error: La versió ha de ser un número.<br>");
					}
				}


				if (!serial.isEmpty()){
					boolean serialok = true;
					for (int i=0; i<serial.length(); i++){
						if(!Character.isDigit(serial.charAt(i))){
							serialok = false;
						}
					}
					if (serialok){
						qserial = Integer.parseInt(serial);
					}else{
						error = true;
						msg = msg.concat("Error: El número de sèrie ha de ser un número.<br>");
					}
				}
				
				if (admin != null && admin.equals("on")){
					qadmin = true;
				}
				
				if (revocat != null && revocat.equals("on")){
					qrev = true;
				}

				GregorianCalendar cal = new GregorianCalendar();
				
				qnb = comprovaData(nbdia, nbmes, nbany);
				if (qnb == null){
					if(!nbdia.isEmpty() || !nbmes.isEmpty() || !nbany.isEmpty()){
						error = true;
						msg = msg.concat("Error: La primera data (NotBefore) no és vàlida.<br>");
					}
				}
				
				qna = comprovaData(nadia, names, naany);
				if (qna == null){
					if(!nadia.isEmpty() || !names.isEmpty() || !naany.isEmpty()){
						error = true;
						msg = msg.concat("Error: La segona data (NotAfter) no és vàlida.<br>");
					}
				}else{
					// Afegim la última hora del dia (23:59:59) perquè compti aquest dia
					cal.setTime(qna);
					cal.add(Calendar.HOUR_OF_DAY, 23);
					cal.add(Calendar.MINUTE, 59);
					cal.add(Calendar.SECOND, 59);
					qna = cal.getTime();	
				}
				
				// Comprovem que la data notBefore sigui anterior a notAfter
				if(qnb != null && qna != null){
					if(!qnb.before(qna)){
						error = true;
						msg = msg.concat("Error: La segona data no pot ser anterior a la primera.<br>");
					}
				}

				if(!email.isEmpty()){
					if(!email.contains("@") || !email.contains(".")){
						// Error l'adreça de correu no es correcta
						error = true;
						msg = msg.concat("Error: Sembla que l'adreça de correu no es correcta. <br>");
					}
				}

				if(!error){
					List<String[]> al = Dbops.cercaCerts(qversio, qserial, qadmin, qrev, qnb, qna, country, st, loc, org, orgunit, cn, email);
					request.setAttribute("rescerca", al);
					desti = "/jsp/rescercacerts.jsp";
				}else{
					request.setAttribute("error", msg);
					desti = "/jsp/cercacerts.jsp";
				}

			}


			if(opcio.equals("cercacrl")){
				// Declarem paràmetres de la cerca
				// Els camps que no volguem fer servir els passarem 
				// negatius si són int o null si són Date.
				int qserial = -1;
				int qmotiu = -1;
				Date qnb = null;
				Date qna = null;		

				// Recollim i comprovem els paràmetres de cerca
				String serial = request.getParameter("serial");
				String nbdia = request.getParameter("nbdia");
				String nbmes = request.getParameter("nbmes");
				String nbany = request.getParameter("nbany");
				String nadia = request.getParameter("nadia");
				String names = request.getParameter("names");
				String naany = request.getParameter("naany");
				String motiu = request.getParameter("motiu");


				if (!serial.isEmpty()){
					boolean serialok = true;
					for (int i=0; i<serial.length(); i++){
						if(!Character.isDigit(serial.charAt(i))){
							serialok = false;
						}
					}
					if (serialok){
						qserial = Integer.parseInt(serial);
					}else{
						error = true;
						msg = msg.concat("El número de sèrie ha de ser un número.<br>");
					}
				}

				GregorianCalendar cal = new GregorianCalendar();
				
				qnb = comprovaData(nbdia, nbmes, nbany);
				if (qnb == null){
					if(!nbdia.isEmpty() || !nbmes.isEmpty() || !nbany.isEmpty()){
						error = true;
						msg = msg.concat("Error: La primera data (NotBefore) no és vàlida.<br>");
					}
				}
				
				qna = comprovaData(nadia, names, naany);
				if (qna == null){
					if(!nadia.isEmpty() || !names.isEmpty() || !naany.isEmpty()){
						error = true;
						msg = msg.concat("Error: La segona data (NotAfter) no és vàlida.<br>");
					}
				}else{
					// Afegim la última hora del dia (23:59:59) perquè compti aquest dia
					cal.setTime(qna);
					cal.add(Calendar.HOUR_OF_DAY, 23);
					cal.add(Calendar.MINUTE, 59);
					cal.add(Calendar.SECOND, 59);
					qna = cal.getTime();	
				}

				// Comprovem que la data notBefore sigui anterior a notAfter
				if(qnb != null && qna != null){
					if(!qnb.before(qna)){
						error = true;
						msg = msg.concat("Error: La segona data no pot ser anterior a la primera.<br>");
					}
				}

				motiu = motiu.substring(4);
				qmotiu = Integer.parseInt(motiu);
				if(qmotiu == 100){
					qmotiu = -1;
				}

				if (error){
					request.setAttribute("error", msg);
					desti = "/jsp/cercacrl.jsp";
				}else{
					List<String[]> al = Dbops.cercaCRL(qserial, qnb, qna, qmotiu);
					request.setAttribute("rescerca", al);
					desti = "/jsp/rescercacerts.jsp";
					if(!msg.isEmpty()){
						request.setAttribute("noti", msg);
					}
				}
			}

		}else{
			desti = "/jsp/sessionerror.jsp";
		}

		RequestDispatcher rd = getServletContext().getRequestDispatcher(desti);
		rd.forward(request, response);

	}
	
	/**
	 * Mètode auxiliar que comprova que les dates passades com a String en 
	 * format dd-mm-aaaa siguin correctes i retorna un objecte Date amb la
	 * data, o null si algún paràmetre és incorrecte o buit.
	 * 
	 * @param dia String que conté el número de dia del mes
	 * @param mes String que conté el número de mes de l'any
	 * @param any String que conté el número de l'any
	 * @return Objecte Date que conté la data o null si és incorrecte o buit
	 */
	private static Date comprovaData(String dia, String mes, String any){
		Date data = null;
		int numDia = -1;
		int numMes = -1;
		int numAny = -1;
		boolean campdataok;
		boolean dataok = false;
		GregorianCalendar cal = new GregorianCalendar();
		
		if (!dia.isEmpty() && !mes.isEmpty() && !any.isEmpty()){
			campdataok = true;
			dataok = true;
			for (int i=0; i < dia.length(); i++){
				if(!Character.isDigit(dia.charAt(i))){
					campdataok = false;
				}
			}
			if(campdataok){
				numDia = Integer.parseInt(dia);
			}else{
				dataok = false;
			}

			campdataok = true;
			for (int i=0; i < mes.length(); i++){
				if(!Character.isDigit(mes.charAt(i))){
					campdataok = false;
				}
			}
			if(campdataok){
				numMes = Integer.parseInt(mes);
			}else{
				dataok = false;
			}

			campdataok = true;
			for (int i=0; i < any.length(); i++){
				if(!Character.isDigit(any.charAt(i))){
					campdataok = false;
				}
			}
			if(campdataok){
				numAny = Integer.parseInt(any);
			}else{
				dataok = false;
			}

			if(dataok){
				//Comprovem que les dates són valors reals.
				if (numMes < 1 || numMes > 12 || numDia < 1 || numAny < 1){
					dataok = false;
				}
				if (numMes == 2){
					if(numAny % 4 == 0){
						if(numDia > 29){
							dataok = false;
						}
					}else{
						if(numDia > 28){
							dataok = false;
						}
					}
				}
				//Mesos de Gener, Març, Maig, Juliol, Agost, Octubre, Desembre
				if (numMes == 1 || numMes == 3 || numMes == 5 || numMes == 7 || numMes == 8 || numMes == 10 || numMes == 12){
					if(numDia > 31){
						dataok = false;
					}
				}

				//Mesos d'Abril, Juny, Setembre, Novembre
				if (numMes == 4 || numMes == 6 || numMes == 9 || numMes == 11){
					if(numDia > 30){
						dataok = false;
					}
				}
			}

			if(dataok){
				cal.clear();
				cal.set(numAny, numMes-1, numDia);
				data = cal.getTime();
			}else{
				data = null;
			}
		}
		return data;
	}

}		