/**
 * @author Lluís Moran
 * 
 * TFC Seguretat Informàtica
 * ETIS - UOC
 * 2011
 * 
 */
package cauoc;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Timestamp;
import java.text.Format;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * Classe que proporciona les funcions necessàries
 * per realitzar operacions amb la base de dades de l'aplicació
 */
public class Dbops {
	static final String DB_DRIVER = "com.mysql.jdbc.Driver";
	static final String DB_URL = "jdbc:mysql://localhost:3306/";

	static final String DB_USER = LoadConfig.DB_USER;
	static final String DB_PASSWORD = LoadConfig.DB_PASSWORD;

	static Connection conn = null;
	static Statement stmt = null;


	/**
	 * Mètode que crea una connexió amb el gestor de bases de dades.
	 * No obre cap base de dades.
	 * 
	 * @return true si es pot fer la connexió.
	 */
	public static boolean conectardb(){
		boolean ret = false;
		try{
			Class.forName(DB_DRIVER);
			conn = DriverManager.getConnection(DB_URL, DB_USER, DB_PASSWORD);
			stmt = conn.createStatement();
			ret = true;
		}catch(SQLException se){
			se.printStackTrace();
		}catch(Exception e){
			e.printStackTrace();
		}

		return ret;
	}

	/**
	 * Mètode que tanca una connexió amb el gestor de bases de dades.
	 * 
	 * @return true si es pot fer la desconnexió.
	 */
	public static boolean desconectardb(){
		boolean ret = false; // Retornem fals si la desconnexió no es pot dur a terme
		try{
			stmt.close();
			conn.close();
			ret = true;
		}catch(SQLException se){
			se.printStackTrace();
		}catch(Exception e){
			e.printStackTrace();
		}finally{
			try{
				if(stmt != null){
					stmt.close();
				}
			}catch(SQLException se2){
			}
			try{
				if(conn != null){
					conn.close();
					ret = true;
				}
			}catch(SQLException se){
				se.printStackTrace();
			}
		}
		return ret;
	}

	/**
	 * Mètode que comprova si la base de dades de l'aplicació existeix.
	 * 
	 * @return true si existeix la base de dades.
	 */
	public static boolean existeixdb(){
		boolean existeix = false;

		if(conectardb()){
			try {
				if (conn != null){
					String sql = "USE magatzem";
					stmt.executeUpdate(sql);;
					existeix = true;
				}
			} catch (SQLException e) {
				// No fem res. Ja es retornarà fals.
			}finally{
				desconectardb();
			}
		}
		return existeix;	
	}

	/**
	 * Mètode que crea la base de dades de l'aplicació
	 * incloent totes les taules necessàries.
	 * Només ho fa en cas que no existeixi la base de dades.
	 * 
	 * @return True si s'ha pogut crear la base de dades
	 */
	public static boolean creardb(){

		boolean ret = false;
		if (existeixdb()){
			System.out.println("La db ja existeix");
		}else{
			try {
				conectardb();
				String sql = "CREATE DATABASE magatzem";
				stmt.executeUpdate(sql);

				sql = "USE magatzem";
				stmt.executeUpdate(sql);

				// Creem l'estructura de la DB
				sql = "CREATE TABLE certificats ("
					+ "Version integer(2), "
					+ "SerialNumber integer(50) NOT NULL, "
					+ "Admin bool DEFAULT False, "
					+ "Rev bool DEFAULT False, "
					+ "SigAlgName varchar(50), "
					+ "IssuerName varchar(200), "
					+ "NotBefore datetime, "
					+ "NotAfter datetime, "
					+ "SubjectName varchar(200), "
					+ "PRIMARY KEY (SerialNumber) "
					+ ")";
				stmt.executeUpdate(sql);

				sql = "CREATE TABLE CRL ("
					+ "SerialNumber integer(50) NOT NULL, "
					+ "IssuerName varchar(200), "
					+ "ThisUpdate datetime, "
					+ "NextUpdate datetime, "
					+ "PRIMARY KEY (SerialNumber) "
					+ ")";
				stmt.executeUpdate(sql);

				sql = "CREATE TABLE CRLcerts ("
					+ "SerialNumber integer(50) NOT NULL, "
					+ "RevocationDate datetime, "
					+ "RevocationReason varchar(100), "
					+ "PRIMARY KEY (SerialNumber) "
					+ ")";
				stmt.executeUpdate(sql);

				ret = true;
			} catch (SQLException e) {
				e.printStackTrace();
			}finally{
				desconectardb();
			}
		}
		return ret;
	}
	
	/**
	 * Mètode que esborra la base de dades de l'aplicació.
	 * Només ho fa en cas que existeixi la base de dades.
	 * 
	 * @return True si s'ha pogut esborrar la base de dades o no existeix
	 */
	public static boolean esborrardb(){
		boolean ret = false;

		if (existeixdb()){
			try {
				conectardb();
				String sql = "DROP DATABASE magatzem";
				stmt.executeUpdate(sql);
				ret = true;
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}else{
			ret = true; // La base de dades no existeix per tant és com si l'haguessim esborrat
		} 

		desconectardb();

		return ret;
	}

	/**
	 * Mètode que cerca a la base de dades de certificats el que té el 
	 * número de sèrie més alt i retorna el següent número de sèrie
	 * com a int.
	 * 
	 * @return int que representa el següent número de sèrie que s'ha de fer servir
	 * per crear un nou certificat.
	 */
	public static int getNextSerial(){
		int serial = 0;
		if (existeixdb()){
			try {
				conectardb();
				String sql = "USE magatzem";
				stmt.executeUpdate(sql);

				sql = "SELECT SerialNumber FROM certificats ORDER BY SerialNumber DESC";
				ResultSet rs = stmt.executeQuery(sql);
				// Accedim a la primera fila on hi haurà l'últim serial que hem fet servir

				if(!rs.next()){
					// Si la taula està buida retornarà serial = 0
					serial = 0;
				}else{
					rs.first();
					serial = rs.getInt("SerialNumber");
				}
				rs.close();
				stmt.close();

			} catch (SQLException e) {
				e.printStackTrace();
			}finally{
				serial = serial + 1; // Incrementem el número de sèrie
				desconectardb();
			}
		}else{
			System.out.println("La Base de Dades no existeix");
		}

		return serial;
	}

	/**
	 * Mètode que guarda les dades d'un certificat a la base de 
	 * dades de certificats.
	 * 
	 * @param version  int que representa la versió del certificat
	 * @param serialNumber  int que representa el número de sèrie
	 * @param sigAlgName  String que representa el nom de l'algorisme de signatura
	 * @param issuerName  String que representa el Distinguished Name de l'emisor del certificat
	 * @param cerNB Date que representa la data d'inici de validesa
	 * @param cerNA Date que representa la data de final de validesa
	 * @param subjectName String que representa el Distinguished Name del subjecte del certificat
	 * 
	 * @return True si s'ha pogut guardar el certificat
	 */
	public static boolean guardaCertificat(int version, 
			int serialNumber, 
			String sigAlgName, 
			String issuerName,
			Date cerNB,
			Date cerNA,
			String subjectName){
		boolean ret = false;
		if (existeixdb()){
			try {

				conectardb();
				String sql = "USE magatzem";
				stmt.executeUpdate(sql);

				Format formatdata;
				formatdata = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
				String notBeforeString = formatdata.format(cerNB);
				String notAfterString = formatdata.format(cerNA);
				
				// En crear-los tots els certificats seran de no Administrador i no revocats
				boolean admin = false;
				boolean rev = false;


				sql = "INSERT INTO certificats VALUES (" + version
				+ ", " + serialNumber
				+ ", " + admin
				+ ", " + rev
				+ ", '" + sigAlgName
				+ "', '" + issuerName
				+ "', '" + notBeforeString
				+ "', '" + notAfterString
				+ "', '" + subjectName + "')";
				stmt.executeUpdate(sql);
				stmt.close();

				ret = true;
			} catch (SQLException e) {
				e.printStackTrace();
			}finally{
				desconectardb();
			}
		}else{
			System.out.println("La Base de Dades no existeix");
		}

		return ret;
	}
	
	/**
	 * Mètode que marca un certificat com a administrador.
	 * 
	 * @param serialNumber Objecte int que representa el número de sèrie del certificat.
	 * @return True si s'aconssegueix marcar.
	 */
	public static boolean ferAdmin(int serialNumber){
		boolean ret = false;
		
		if (existeixdb()){
			try {
				conectardb();
				String sql = "USE magatzem";
				stmt.executeUpdate(sql);

				sql = "UPDATE certificats SET Admin = true	WHERE SerialNumber = " + serialNumber;
				stmt.executeUpdate(sql);
				stmt.close();
				ret = true;

			} catch (SQLException e) {
				// No fem res. Si hi ha una excepció retornarà fals.
			}finally{
				desconectardb();
			}
		}else{
			System.out.println("La Base de Dades no existeix");
			// No fem res. Es retornarà False conforme hi ha un error.
		}
	
		return ret;
	}
	
	/**
	 * Mètode que marca un certificat per a que deixi de ser administrador .
	 * 
	 * @param serialNumber Objecte int que representa el número de sèrie del certificat.
	 * @return True si s'aconssegueix desmarcar.
	 */
	public static boolean revocarAdmin(int serialNumber){
		boolean ret = false;
		
		if (existeixdb()){
			try {
				conectardb();
				String sql = "USE magatzem";
				stmt.executeUpdate(sql);

				sql = "UPDATE certificats SET Admin = false	WHERE SerialNumber = " + serialNumber;
				stmt.executeUpdate(sql);
				stmt.close();
				ret = true;

			} catch (SQLException e) {
				// No fem res. Si hi ha una excepció retornarà fals.
			}finally{
				desconectardb();
			}
		}else{
			System.out.println("La Base de Dades no existeix");
			// No fem res. Es retornarà False conforme hi ha un error.
		}
	
		return ret;
	}
	
	/**
	 * Mètode que comprova si un certificat té l'indicador de revocat.
	 * Retorna True en cas que estigui revocat o no existeixi.
	 * 
	 * @param serialNumber Objecte int que representa el número de sèrie del certificat.
	 * @return True si el certificat està revocat o en cas que no existeixi.
	 */
	public static boolean esRevocat(int serialNumber){
		boolean ret = false;
		
		if (existeixdb()){
			try {
				conectardb();
				String sql = "USE magatzem";
				stmt.executeUpdate(sql);

				sql = "SELECT Rev FROM certificats WHERE SerialNumber = " + serialNumber;
				ResultSet rs = stmt.executeQuery(sql);
				// Accedim a la primera fila on hi haurà només l'estat de revocació del certificat

				if(!rs.next()){
					// Si la taula està buida retornarà true
					ret = true;
				}else{
					rs.first();
					ret = rs.getBoolean("Rev");
				}
				rs.close();
				stmt.close();

			} catch (SQLException e) {
				e.printStackTrace();
			}finally{
				desconectardb();
			}
		}else{
			System.out.println("La Base de Dades no existeix");
			// No fem res. Es retornarà False conforme hi ha un error.
		}
	
		return ret;
	}
	
	/**
	 * Mètode que comprova si un certificat està autoritzat com a Administrador.
	 * 
	 * @param serialNumber Objecte int que representa el número de sèrie del certificat.
	 * @return True si el certificat està marcat com a Administrador.
	 */
	public static boolean esAdmin(int serialNumber){
		boolean ret = false;
		
		if (existeixdb()){
			try {
				conectardb();
				String sql = "USE magatzem";
				stmt.executeUpdate(sql);

				sql = "SELECT Admin FROM certificats WHERE SerialNumber = " + serialNumber;
				ResultSet rs = stmt.executeQuery(sql);
				// Accedim a la primera fila on hi haurà només l'estat d'aministració del certificat

				if(rs.next()){
					rs.first();
					ret = rs.getBoolean("Admin");
				}// Si la taula està buida ja es retornarà false
				rs.close();
				stmt.close();

			} catch (SQLException e) {
				e.printStackTrace();
			}finally{
				desconectardb();
			}
		}else{
			System.out.println("La Base de Dades no existeix");
			// No fem res. Es retornarà False conforme hi ha un error.
		}
	
		return ret;
	}

	/**
	 * Mètode que cerca a la taula de la CRL i retorna el següent número de sèrie
	 * com a int.
	 * 
	 * @return int que representa el següent número de sèrie que s'ha de fer servir
	 * per actualitzar la CRL.
	 */
	public static int getCRLNextSerial(){
		int serial = 0;
		if (existeixdb()){
			try {
				conectardb();
				String sql = "USE magatzem";
				stmt.executeUpdate(sql);

				sql = "SELECT SerialNumber FROM CRL ORDER BY SerialNumber DESC";
				ResultSet rs = stmt.executeQuery(sql);
				// Accedim a la primera fila on hi haurà l'últim serial que hem fet servir

				if(!rs.next()){
					// Si la taula està buida retornarà serial = 0
					serial = 0;
				}else{
					rs.first();
					serial = rs.getInt("SerialNumber");
				}
				rs.close();
				stmt.close();

			} catch (SQLException e) {
				e.printStackTrace();
			}finally{
				serial = serial + 1; // Incrementem el número de sèrie
				desconectardb();
			}
		}else{
			System.out.println("La Base de Dades no existeix");
		}

		return serial;
	}

	/**
	 * Mètode que actualitza la taula CRL en cada actualització de la llista 
	 * 
	 * @param serialNumber  int que representa el número de sèrie de la CRL
	 * @param issuerName  String que representa el Distinguished Name de l'emisor del certificat
	 * @param thisUpdate Date que representa la data de l'actualització
	 * @param nextUpdate Date que representa la data de la següent actualització
	 * 
	 * @return True si s'ha pogut actualitzar la CLR
	 */
	public static boolean actualitzaCRL(int serialNumber, 
			String issuerName,
			Date thisUpdate,
			Date nextUpdate){
		boolean ret = false;
		if (existeixdb()){
			try {
				conectardb();
				String sql = "USE magatzem";
				stmt.executeUpdate(sql);

				Format formatdata;
				formatdata = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
				String thisUpdateString = formatdata.format(thisUpdate);
				String nextUpdateString = formatdata.format(nextUpdate);


				sql = "INSERT INTO CRL VALUES (" + serialNumber
				+ ", '" + issuerName
				+ "', '" + thisUpdateString
				+ "', '" + nextUpdateString + "')";
				stmt.executeUpdate(sql);
				stmt.close();
				
				ret = true;
			} catch (SQLException e) {
				e.printStackTrace();
			}finally{
				desconectardb();
			}
		}else{
			System.out.println("La Base de Dades no existeix");
			// No fem res. Es retornarà False conforme hi ha un error.
		}

		return ret;
	}

	/**
	 * Mètode que guarda les dades d'un certificat revocat 
	 * a la taula de certificats revocats.
	 * 
	 * @param serialNumber  int que representa el número de sèrie
	 * @param revDate Date que representa la data de revocació del certificat
	 * @param revMotiu String que representa el motiu pel que es revoca el certificat
	 * 
	 * @return True si s'ha pogut guardar el certificat
	 */
	public static boolean guardaCertCRL(int serialNumber, Date revDate, int revMotiu){
		boolean ret = false;
		if (existeixdb()){
			try {
				conectardb();
				String sql = "USE magatzem";
				stmt.executeUpdate(sql);

				Format formatdata;
				formatdata = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
				String revDateString = formatdata.format(revDate);

				// Passem el motiu de revocació a String
				String revMotiuString = "";
				if(revMotiu == 0){
					revMotiuString = "Sense especificar";
				}
				if(revMotiu == 1){
					revMotiuString = "Clau compromesa";
				}
				if(revMotiu == 2){
					revMotiuString = "Autoritat Certificadora compromesa";
				}
				if(revMotiu == 3){
					revMotiuString = "Canvi d`afiliació";
				}
				if(revMotiu == 4){
					revMotiuString = "Substituit";
				}
				if(revMotiu == 5){
					revMotiuString = "Cessament d`operacions";
				}
				if(revMotiu == 6){
					revMotiuString = "Certificat retingut";
				}
				if(revMotiu == 8){
					revMotiuString = "Eliminar de CRL";
				}
				if(revMotiu == 9){
					revMotiuString = "Retirada de privilegis";
				}
				if(revMotiu == 10){
					revMotiuString = "Clau d`Autoritat d`Atribut compromesa";
				}

				// Comprovem si la revocació és el cas especial de Eliminar de CRL
				if (revMotiu == 8){
					sql = "SELECT RevocationReason FROM CRLcerts WHERE SerialNumber = " + serialNumber;
					ResultSet rs = stmt.executeQuery(sql);
					if(rs.next()){
						if (rs.getString(1).equals("Certificat retingut")){
							// Eliminem el certificat de la llista CRL
							sql = "DELETE FROM CRLcerts WHERE SerialNumber = " + serialNumber;
							stmt.executeUpdate(sql);

							//Modifiquem la taula de certificats
							sql = "UPDATE certificats SET Rev = False WHERE SerialNumber = " + serialNumber;
							stmt.executeUpdate(sql);
							ret = true;
						}else{
							ret = false;
						}	
					}else{
						ret = false;
					}
				}else{
					sql = "INSERT INTO CRLcerts VALUES (" + serialNumber
					+ ", '" + revDateString
					+ "', '" + revMotiuString + "')";
					stmt.executeUpdate(sql);

					//Modifiquem la taula de certificats
					sql = "UPDATE certificats SET Rev = True WHERE SerialNumber = " + serialNumber;
					stmt.executeUpdate(sql);
					ret = true;
				}
				
				stmt.close();

				
			} catch (SQLException e) {
				ret = false;
				e.printStackTrace();
			}finally{
				desconectardb();
			}
		}else{
			System.out.println("La Base de Dades no existeix");
			// No fem res. Es retornarà False conforme hi ha un error.
		}

		return ret;
	}

	/**
	 * Mètode que fa una cerca dels certificats emesos a la base de dades a partir de 
	 * diferents paràmetres, per retornar el resultat de la cerca com a una llista d'Array d'String.
	 * Cada Array d'String contindrà una filera de la taula de la cerca. 
	 * La primera filera contindrà els noms dels camps (columnes). 
	 * Cada String contindrà el valor de la filera i columna corresponent.
	 * Els paràmetres que estiguin en blanc seran ignorats i la cerca contindrà tots els seus valors.
	 * S'ha d'haver filtrat els paràmetres abans d'invocar el mètode per assegurar que els
	 * valors seran correctes.
	 * 
	 * @param versio La versió del certificat com a int.
	 * @param numSerie El número de sèrie del certificat com a int.
	 * @param admin Boolean true si el certificat té accés d'administrador.
	 * @param revocat Boolean true si el certificat ha estat revocat.
	 * @param notBefore La data abans de la qual el certificat no és vàlid com a Date.
	 * @param notAfter La data desprès de la qual el certificat no és vàlid com a Date.
	 * @param dnC El codi de país del Distinguished Name com a String.
	 * @param dnST L'estat o província del Distinguished Name com a String.
	 * @param dnL La localitat del Distinguished Name com a String.
	 * @param dnO El nom de la organització del Distinguished Name com a String.
	 * @param dnOU El nom de la unitat de la organització del Distinguished Name com a String.
	 * @param dnCN El nom del Distinguished Name com a String.
	 * @param dnEmail L'email del Distinguished Name com a String.
	 * 
	 * @return Objecte List<String[]> que conté una llista d'arrays d'String amb el resultat de la cerca.
	 */
	public static List<String[]> cercaCerts(int versio,
			int numSerie,
			boolean admin,
			boolean revocat,
			Date notBefore,
			Date notAfter,
			String dnC,
			String dnST,
			String dnL,
			String dnO,
			String dnOU,
			String dnCN,
			String dnEmail){

		List<String[]> llista = new ArrayList<String[]>();
		if (existeixdb()){
			try {
				conectardb();
				String sql = "USE magatzem";
				stmt.executeUpdate(sql);

				sql = "SELECT * FROM certificats WHERE ";

				if(versio >= 0){
					sql = sql.concat("Version = " + versio + " AND ");
				}
				
				if(numSerie >= 0){
					sql = sql.concat("SerialNumber = " + numSerie + " AND ");
				}

				if(admin){
					sql = sql.concat("Admin = true AND ");
				}
				
				if(revocat){
					sql = sql.concat("Rev = true AND ");
				}
				
				if(notBefore != null){
					Format formatdata;
					formatdata = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
					String nbString = formatdata.format(notBefore);
					sql = sql.concat("NotBefore >= '" + nbString + "' AND ");
				}
				if(notAfter != null){
					Format formatdata;
					formatdata = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
					String naString = formatdata.format(notAfter);
					sql = sql.concat("NotAfter >= '" + naString + "' AND ");
				}
				if(!dnC.isEmpty()){
					sql = sql.concat("SubjectName LIKE '%C=" + dnC + "%' AND ");
				}
				if(!dnST.isEmpty()){
					sql = sql.concat("SubjectName LIKE '%ST=" + dnST + "%' AND ");
				}
				if(!dnL.isEmpty()){
					sql = sql.concat("SubjectName LIKE '%L=" + dnL + "%' AND ");
				}
				if(!dnO.isEmpty()){
					sql = sql.concat("SubjectName LIKE '%O=" + dnO + "%' AND ");
				}
				if(!dnOU.isEmpty()){
					sql = sql.concat("SubjectName LIKE '%OU=" + dnOU + "%' AND ");
				}
				if(!dnCN.isEmpty()){
					sql = sql.concat("SubjectName LIKE '%CN=" + dnCN + "%' AND ");
				}
				if(!dnEmail.isEmpty()){
					sql = sql.concat("SubjectName LIKE '%E=" + dnEmail + "%' AND ");
				}

				// Elimina l'últim AND de la comanda SQL
				String noAnd = " AND ";
				if(sql.endsWith(noAnd)){
					sql = sql.substring(0, sql.length() - noAnd.length());
				}

				// Si tot és NULL elimina el WHERE de la comanda SQL
				// perquè surti el llistat complert
				String noWhere = " WHERE ";
				if(sql.endsWith(noWhere)){
					sql = sql.substring(0, sql.length() - noWhere.length());
				}

				sql = sql.concat(" ORDER BY SerialNumber ASC");
				ResultSet rs = stmt.executeQuery(sql);
				llista = resultSetToList(rs);

				rs.close();
				stmt.close();

			} catch (SQLException e) {
				e.printStackTrace();
			}finally{
				desconectardb();
			}
		}else{
			System.out.println("La Base de Dades no existeix");
		}

		return llista;
	}

	/**
	 * Mètode que fa una cerca dels certificats revocats a la base de dades a partir de 
	 * diferents paràmetres, per retornar el resultat de la cerca com a una llista d'Array d'String.
	 * Cada Array d'String contindrà una filera de la taula de la cerca. 
	 * La primera filera contindrà els noms dels camps (columnes). 
	 * Cada String contindrà el valor de la filera i columna corresponent.
	 * Els paràmetres que estiguin en blanc seran ignorats i la cerca contindrà tots els seus valors.
	 * S'ha d'haver filtrat els paràmetres abans d'invocar el mètode per assegurar que els
	 * valors seran correctes.
	 * 
	 * @param numSerie El número de sèrie del certificat com a int.
	 * @param notBefore La data abans de la qual el certificat no ha estat revocat com a Date.
	 * @param notAfter La data desprès de la qual el certificat no ha estat revocat com a Date.
	 * @param revMotiu El codi del motiu de revocació com a int.
	 * 
	 * @return Objecte List<String[]> que conté una llista d'arrays d'String amb el resultat de la cerca.
	 */
	public static List<String[]> cercaCRL(int numSerie, Date notBefore, Date notAfter, int revMotiu){

		List<String[]> llista = new ArrayList<String[]>();
		if (existeixdb()){
			try {
				conectardb();
				String sql = "USE magatzem";
				stmt.executeUpdate(sql);

				sql = "SELECT * FROM CRLcerts WHERE ";

				if(numSerie >= 0){
					sql = sql.concat("SerialNumber = " + numSerie + " AND ");
				}

				if(notBefore != null){
					Format formatdata;
					formatdata = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
					String nbString = formatdata.format(notBefore);
					sql = sql.concat("RevocationDate >= '" + nbString + "' AND ");
				}
				if(notAfter != null){
					Format formatdata;
					formatdata = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
					String naString = formatdata.format(notAfter);
					sql = sql.concat("RevocationDate >= '" + naString + "' AND ");
				}

				if(revMotiu >= 0){
					String revMotiuString = "";
					if(revMotiu == 0){
						revMotiuString = "Sense especificar";
					}
					if(revMotiu == 1){
						revMotiuString = "Clau compromesa";
					}
					if(revMotiu == 2){
						revMotiuString = "Autoritat Certificadora compromesa";
					}
					if(revMotiu == 3){
						revMotiuString = "Canvi d`afiliació";
					}
					if(revMotiu == 4){
						revMotiuString = "Substituit";
					}
					if(revMotiu == 5){
						revMotiuString = "Cessament d`operacions";
					}
					if(revMotiu == 6){
						revMotiuString = "Certificat retingut";
					}
					if(revMotiu == 8){
						revMotiuString = "Eliminar de CRL";
					}
					if(revMotiu == 9){
						revMotiuString = "Retirada de privilegis";
					}
					if(revMotiu == 10){
						revMotiuString = "Clau d`Autoritat d`Atribut compromesa";
					}

					sql = sql.concat("RevocationReason = '" + revMotiuString + "' AND ");
				}


				// Elimina l'últim AND de la comanda SQL
				String noAnd = " AND ";
				if(sql.endsWith(noAnd)){
					sql = sql.substring(0, sql.length() - noAnd.length());
				}

				// Si tot és NULL elimina el WHERE de la comanda SQL
				// perquè surti el llistat complert
				String noWhere = " WHERE ";
				if(sql.endsWith(noWhere)){
					sql = sql.substring(0, sql.length() - noWhere.length());
				}

				sql = sql.concat(" ORDER BY SerialNumber ASC");
				ResultSet rs = stmt.executeQuery(sql);
				llista = resultSetToList(rs);

				rs.close();
				stmt.close();

			} catch (SQLException e) {
				e.printStackTrace();
			}finally{
				desconectardb();
			}
		}else{
			System.out.println("La Base de Dades no existeix");
		}

		return llista;
	}

	/**
	 * Mètode que transforma un ResultSet obtingut d'una cerca a la base de dades en
	 * una llista d'arrays d'String per passar el resultat i mostrar-lo.
	 * Com que l'objectiu de les dades és ser mostrat o imprés, es passaran tots els 
	 * resultats com a String.
	 * 
	 * @param rs Objecte ResultSet amb el contingut d'una cerca
	 * @return Objecte List<String[]> que conté una llista d'arrays d'String amb el contingut del ResultSet
	 */
	private static List<String[]> resultSetToList(ResultSet rs){
		List<String[]> llista = new ArrayList<String[]>();

		try{
			int numColumnes = rs.getMetaData().getColumnCount();

			// Guardem la capçalera
			String cap[] = new String[numColumnes];
			for (int j=1; j <= numColumnes; j++){
				cap[j-1] = rs.getMetaData().getColumnName(j);
			}
			llista.add(cap);

			while (rs.next()){
				String fila[] = new String[numColumnes];

				for (int i=1; i <= numColumnes; i++){
					String tipus = rs.getMetaData().getColumnTypeName(i);

					if(tipus == "INT"){
						fila[i-1] = String.valueOf(rs.getInt(i));
					}
					
					if(tipus == "TINYINT"){
						if (rs.getBoolean(i)){
							fila[i-1] = "Sí";
						}else{
							fila[i-1] = "No";
						}
					}

					if(tipus == "DATETIME"){
						Timestamp data = rs.getTimestamp(i);
						Format formatdata;
						formatdata = new SimpleDateFormat("dd-MM-yyyy HH:mm:ss");
						String dataString = formatdata.format(data);
						fila[i-1] = dataString;
					}

					if(tipus == "VARCHAR"){
						fila[i-1] = rs.getString(i);
					}
				}

				llista.add(fila);
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}

		return llista;
	}
}