/**
 * @author Lluís Moran
 * 
 * TFC Seguretat Informàtica
 * ETIS - UOC
 * 2011
 * 
 */
package cauoc;

import java.io.IOException;
import java.security.cert.X509Certificate;

import javax.servlet.RequestDispatcher;
import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;



/**
 * Servlet que proporciona la interfície d'autenticació d'administrador
 * i les funcions públiques de l'aplicació.
 *
 */
@WebServlet("/login")
public class Login extends HttpServlet{
		
	protected void doPost (HttpServletRequest request, HttpServletResponse response)
		throws ServletException, IOException{
		String desti = "";
		
		// Passem el paràmetre "opcio" a int per poder fer servir un switch
		String opcio = request.getParameter("opcio");
		int op = 0;
		if(opcio.equals("pkcs10")){
			op = 1;
		}
		if(opcio.equals("pkcs12")){
			op = 2;
		}
		if(opcio.equals("admin")){
			op = 3;
		}
		if(opcio.equals("logincert")){
			op = 4;
		}
		
		switch (op) {
		case 1:
			desti = "/jsp/menupkcs10.jsp";
			break;
		case 2:
			desti = "/jsp/menupkcs12.jsp";
			break;
		case 3:
			// Comprova que el login és local 
			if(request.getLocalAddr().equals(request.getRemoteAddr())){
				// Recopilació de dades del formulari
				String usr = request.getParameter("user");
				String pwd = request.getParameter("password");

				// Comprovació de login vàlid
				if(loginValid(usr, pwd)){
					//Activa la comprovació de sessió
					HttpSession session = request.getSession(true);
					session.setAttribute("admin", true);
					
					if(session.getAttribute("admin") != null && session.getAttribute("admin").equals(true)){
						desti = "/jsp/menuadmin.jsp";
					}else{
						desti = "/jsp/sessionerror.jsp";
					}
				}else{
					desti = "/jsp/sessionerror.jsp";
				}
			}else{
				desti = "/jsp/localerror.jsp";
			}

			break;
		case 4:
			String path = getServletContext().getRealPath("");
			String msg = "";
			if(request.getAttribute ("javax.servlet.request.X509Certificate") != null){
				
	
				
				X509Certificate[] adminChain = (X509Certificate[])request.getAttribute("javax.servlet.request.X509Certificate");
				X509Certificate adminCert = adminChain[0];
				msg = Caops.adminCert(path, adminCert);
				if(msg != null && !msg.startsWith("Error")){
					//Activem sessió
					HttpSession session = request.getSession(true);
					session.setAttribute("admin", true);

					//Comprovem que s'ha activat i saltem a la pàgina corresponent
					if(session.getAttribute("admin") != null && session.getAttribute("admin").equals(true)){
						desti = "/jsp/menuadmin.jsp";
					}else{
						desti = "/jsp/sessionerror.jsp";
					}
				}else{
					request.setAttribute("error", msg);
					desti = "/jsp/sessionerror.jsp";
				}
			}else{
				msg = "Error: No heu presentat cap certificat. <br>";
				request.setAttribute("error", msg);
				desti = "/jsp/sessionerror.jsp";
			}

			break;	
		}

		RequestDispatcher rd = getServletContext().getRequestDispatcher(desti);
		rd.forward(request, response);
	}
	
	/**
	 * Mètode que comprova que les dades de login siguin correctes.
	 * 
	 * @param usr Nom d'usuari
	 * @param pwd Contrassenya
	 * @return True si les dades són vàlides
	 */
	protected boolean loginValid (String usr, String pwd){
		boolean ret = false;
		String user = LoadConfig.LOCAL_USER;
		String password = LoadConfig.LOCAL_PWD;
		
		if (usr.equals(user) & pwd.equals(password)){
			ret = true;
		}
		return ret;
	}

}
