/**
 * @author Lluís Moran
 * 
 * TFC Seguretat Informàtica
 * ETIS - UOC
 * 2011
 * 
 */
package cauoc;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import javax.servlet.RequestDispatcher;
import javax.servlet.ServletException;
import javax.servlet.annotation.MultipartConfig;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;



/**
 * Servlet que recull les dades del menú d'usuari
 * i realitza la funció demanada mitjançant les
 * classes internes de l'aplicació.
 */
@WebServlet("/public")
@MultipartConfig
public class Public extends HttpServlet {

	protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {

		String msg = "";
		
		boolean error = false; //determina si s'ha de mostrar un missatge d'error 
		String path = getServletContext().getRealPath("");

		String opcio = request.getParameter("opcio");
		if(opcio.equals("pkcs10")){

			InputStream is = request.getPart("fitxer").getInputStream();

			String pathCert = Caops.signapkcs10(path, is);
			if(pathCert != null && !pathCert.startsWith("Error")){
				File fitxer = new File(pathCert);
				response.setContentType("aplication/x-download");
				response.setHeader("Content-Disposition", "attachment; filename=signedcert.cer");
				response.setContentLength((int)fitxer.length());

				FileInputStream fis = new FileInputStream(pathCert);
				byte[] buffer = new byte[(int) fitxer.length()];
				fis.read(buffer);
				fis.close();

				OutputStream os = response.getOutputStream();
				os.write(buffer);
				os.close();
			}else{
				error = true;
				msg = pathCert;
			}

			if(error){
				request.setAttribute("error", msg);
				String desti = "/jsp/menupkcs10.jsp";
				RequestDispatcher rd = getServletContext().getRequestDispatcher(desti);
				rd.forward(request, response);
			}
		}

		if(opcio.equals("pkcs12")){
			String dnC = request.getParameter("C");
			String dnST = request.getParameter("ST");
			String dnL = request.getParameter("L");
			String dnO = request.getParameter("O");
			String dnOU = request.getParameter("OU");
			String dnCN = request.getParameter("CN");
			String dnEmail = request.getParameter("EMAILADDRESS");
			String dnPwd = request.getParameter("password");
			String dnPwd2 = request.getParameter("password2");

			String dn = "";
			if(!dnEmail.isEmpty()){
				if(dnEmail.contains("@") && dnEmail.contains(".")){
					dn = dn + "EMAILADDRESS=" + dnEmail + ", ";
				}else{
					// Error l'adreça de correu no es correcta
					error = true;
					msg = msg.concat("Error: Sembla que l'adreça de correu no es correcta. <br>");
				}
			}

			if(!dnCN.isEmpty()){
				dn = dn + "CN=" + dnCN + ", ";
			}else{
				// Error CN és obligatori
				error = true;
				msg = msg.concat("Error: El camp CN no pot estar buit. <br>");
			}

			if(!dnOU.isEmpty()){
				dn = dn + "OU=" + dnOU + ", ";
			}

			if(!dnO.isEmpty()){
				dn = dn + "O=" + dnO + ", ";
			}else{
				// Error O és obligatori
				error = true;
				msg = msg.concat("Error: El camp O no pot estar buit. <br>");
			}

			if(!dnL.isEmpty()){
				dn = dn + "L=" + dnL + ", ";
			}

			if(!dnST.isEmpty()){
				dn = dn + "ST=" + dnST + ", ";
			}else{
				// Error ST és obligatori
				error = true;
				msg = msg.concat("Error: El camp ST no pot estar buit. <br>");
			}

			if(!dnC.isEmpty()){
				dn = dn + "C=" + dnC;
			}else{
				// Error C és obligatori
				error = true;
				msg = msg.concat("Error: El camp C no pot estar buit. <br>");
			}

			if(dnPwd.isEmpty() || dnPwd2.isEmpty()){
				// Error password buit
				error = true;
				msg = msg.concat("Error: Els camps de Contrassenya no poden estar buits. <br>");
			}else{
				// Fem que la contrassenya no superi els 7 caràcters per evitar que 
				// s'hagi d'instal·lar el Java Cryptography Extension (JCE) Unlimited 
				// Strength Jurisdiction Policy Files que limita la mida de les claus per
				// motius de polítiques i controls d'exportació dels Estats Units.
				if(dnPwd.length() > 7 || dnPwd2.length() > 7){
					// error password massa gran
					error = true;
					msg = msg.concat("Error: Les contrassenyes no poden tenir més de 7 caràcters. <br>");
				}else{
					if(!dnPwd.equals(dnPwd2)){
						// Error els passwords no són iguals
						error = true;
						msg = msg.concat("Error: Les contrassenyes no coincideixen. <br>");
					}
				}
			}

			if(!error){
				String pathCert = Caops.generapkcs12(dn, dnPwd, path);

				if(pathCert != null && !pathCert.startsWith("Error")){
					File fitxer = new File(pathCert);
					response.setContentType("aplication/x-download");
					response.setHeader("Content-Disposition", "attachment; filename=" + dnCN.replace(" ", "_") + ".p12");
					response.setContentLength((int)fitxer.length());

					FileInputStream fis = new FileInputStream(pathCert);
					byte[] buffer = new byte[(int) fitxer.length()];
					fis.read(buffer);
					fis.close();

					OutputStream os = response.getOutputStream();
					os.write(buffer);
					os.close();
				}else{
					error = true;
					msg = pathCert;
				}
			}else{
				request.setAttribute("error", msg);
				String desti = "/jsp/menupkcs12.jsp";
				RequestDispatcher rd = getServletContext().getRequestDispatcher(desti);
				rd.forward(request, response);
			}
		}
	}
}		
