#!/bin/sh
#
#
#Script per generar els certificats i claus d'una CA
#i el certificat de Tomcat per a la connexió SSL
#de l'aplicació
#
#Basat en l'script de:
# Tim Hudson
# tjh@cryptsoft.com
#
#Que es pot trobar a la documentació de OpenSSL
#/usr/lib/ssl/misc/CA.sh
#
# CA -mysql ... Genera un usuari de mySQL
# CA -novaca ... Genera una carpeta amb la CA
# CA -tomcat ... Genera una carpeta amb el keystore per a Tomcat 
#
#



if [ -z "$OPENSSL" ]; then OPENSSL=openssl; fi

DAYS="-days 365"	# 1 any
CADAYS="-days 1825"	# 3 anys
REQ="$OPENSSL req $SSLEAY_CONFIG"
CA="$OPENSSL ca $SSLEAY_CONFIG"
VERIFY="$OPENSSL verify"
X509="$OPENSSL x509"
PKCS8="$OPENSSL pkcs8"
CRL="$OPENSSL crl"

CATOP=../ca
CAKEY=./cakey.pem
CAKEYDER=./cakey.der
CAREQ=./careq.pem
CACERT=./cacert.pem
CACRL=./capem.crl
CACRLDER=./ca.crl
CAKS=./ca.jks
CNF=./ca.cnf
TOMCAT=../tomcat
PUB=../pub

# Recull de variables de configuració d'usuari (user.conf)
capassword=`sed '/^\#/d' user.conf | grep 'capassword' | tail -n 1 | cut -d "=" -f2- | sed 's/^[[:space:]]*//;s/[[:space:]]*$//'`
tomcatpassword=`sed '/^\#/d' user.conf | grep 'tomcatpassword' | tail -n 1 | cut -d "=" -f2- | sed 's/^[[:space:]]*//;s/[[:space:]]*$//'`
mysqluser=`sed '/^\#/d' user.conf | grep 'mysqluser' | tail -n 1 | cut -d "=" -f2- | sed 's/^[[:space:]]*//;s/[[:space:]]*$//'`
mysqlpassword=`sed '/^\#/d' user.conf | grep 'mysqlpassword' | tail -n 1 | cut -d "=" -f2- | sed 's/^[[:space:]]*//;s/[[:space:]]*$//'`

for i
do
case $i in
-\?|-h|-help)
    echo "opcions: CAuoc -novaca|-tomcat|-mysql|" >&2
    exit 0
    ;;
-novaca)     
    # if explicitly asked for or it doesn't exist then setup the directory
    NEW="1"
    if [ "$NEW" -o ! -f ${CATOP}/serial ]; then
	# Creem la jerarquia de carpetes
	mkdir ${CATOP} 
	mkdir ${CATOP}/certificats 
	mkdir ${CATOP}/crl 
	mkdir ${CATOP}/privat
	mkdir ${PUB}
	echo "00" > ${CATOP}/serial
        echo "00" > ${CATOP}/crlnumber
	touch ${CATOP}/index.txt
    fi
    if [ ! -f ${CATOP}/private/$CAKEY ]; then
        echo "Creant el certificat de la CA ..."
	$REQ -x509 -newkey rsa:2048 $CADAYS \
       	    -keyout ${CATOP}/privat/$CAKEY \
            -nodes \
            -passout pass:$capassword \
            -out ${CATOP}/$CACERT \
            -extensions v3_ca \
            -config $CNF

	# Copiem el certificat a la carpeta pub
	cp ${CATOP}/$CACERT ${PUB}
 
        keytool -importcert -alias ca -file ${CATOP}/$CACERT \
            -keypass $capassword \
            -keystore ${CATOP}/$CAKS -storepass $capassword 

	# Passem la clau a format DER
	$PKCS8 -topk8 -nocrypt \
           -in ${CATOP}/privat/$CAKEY -inform PEM \
           -out ${CATOP}/privat/$CAKEYDER -outform DER

        $CA -gencrl -out ${CATOP}/crl/$CACRL -config $CNF

        # Passem la CRL a format DER
        $CRL -in ${CATOP}/crl/$CACRL -inform PEM -out ${CATOP}/crl/$CACRLDER -outform DER

	# Copiem la CRL a la carpeta pub
	cp ${CATOP}/crl/$CACRLDER ${PUB}

	RET=$?
    fi
    ;;
-tomcat) 
    NEW="1"
    if [ "$NEW" -o ! -f ${CATOP}/serial ]; then
	# Creem la carpeta de tomcat
	mkdir ${TOMCAT} 
    fi
    echo "Creant el certificat de Tomcat ..."
    echo "Premeu Enter quan us demani si voleu la mateixa contrassenya..."
    keytool -genkey -alias tomcat -keyalg RSA \
            -dname "CN=Tomcat, OU=ETIS, O=UOC, L=Barcelona, ST=Barcelona, C=ES" \
            -keystore ${TOMCAT}/tomcatks.jks -storepass $tomcatpassword
        
    keytool -certreq -keyalg RSA -alias tomcat \
            -file ${TOMCAT}/certreq.csr -keystore ${TOMCAT}/tomcatks.jks \
	    -storepass $tomcatpassword 

    $CA $DAYS -cert  ${CATOP}/$CACERT -keyfile ${CATOP}/privat/$CAKEY \
            -passin pass:$capassword \
            -out ${TOMCAT}/tomcatssl.pem \
	    -outdir ${CATOP}/certificats \
            -in ${TOMCAT}/certreq.csr -config $CNF

    keytool -import -alias root -keystore ${TOMCAT}/tomcatks.jks \
            -storepass $tomcatpassword \
            -trustcacerts -file ${CATOP}/cacert.pem 

    keytool -import -alias caroot -keystore ${TOMCAT}/trustedca.jks \
            -storepass $tomcatpassword \
            -trustcacerts -file ${CATOP}/cacert.pem 

    $X509 -in ${TOMCAT}/tomcatssl.pem -out ${TOMCAT}/tomcatssl.der -outform DER 

    keytool -import -alias tomcat -keystore ${TOMCAT}/tomcatks.jks \
            -storepass $tomcatpassword \
            -file ${TOMCAT}/tomcatssl.der 
 
    RET=$?

    echo "S'ha creat el fitxer tomcatks.jks i el trustedca.jks a la carpeta tomcat."
    echo "Copieu-los a la carpeta d'instal·lació de Tomcat 7 i "
    echo "seguiu les instruccions de la documentació per modificar"
    echo "els fitxers de configuració." 
    ;;
-mysql)
    echo "Creant l'usuari de mySQL..."
    echo "Introduiu la contrassenya de root"
    mysql -u root -p -e "GRANT ALL ON *.* TO $mysqluser@localhost IDENTIFIED BY '$mysqlpassword';"
    
    RET=$?
    ;;
*)
    echo "Unknown arg $i";
    exit 1
    ;;
esac
done
exit $RET

